@@ -21,10 +21,9 @@ import sys
 import attr
 import aws_encryption_sdk
 import six
-from aws_encryption_sdk.materials_managers import CommitmentPolicy
+from aws_encryption_sdk.materials_managers import CommitmentPolicy  # noqa pylint: disable=unused-import
 from base64io import Base64IO
 
-from aws_encryption_sdk_cli.internal.arg_parsing import _is_decrypt_mode
 from aws_encryption_sdk_cli.internal.identifiers import OUTPUT_SUFFIX, OperationResult
 from aws_encryption_sdk_cli.internal.logging_utils import LOGGER_NAME
 from aws_encryption_sdk_cli.internal.metadata import MetadataWriter, json_ready_header, json_ready_header_auth
@@ -154,7 +153,6 @@ class IOHandler(object):
     :param bool no_overwrite: Should never overwrite existing files
     :param bool decode_input: Should input be base64 decoded before operation
     :param bool encode_output: Should output be base64 encoded after operation
-    :param bool buffer_output: Should buffer entire output before releasing to destination
     :param dict required_encryption_context: Encryption context key-value pairs to require
     :param list required_encryption_context_keys: Encryption context keys to require
     """
@@ -164,14 +162,12 @@ class IOHandler(object):
     no_overwrite = attr.ib(validator=attr.validators.instance_of(bool))
     decode_input = attr.ib(validator=attr.validators.instance_of(bool))
     encode_output = attr.ib(validator=attr.validators.instance_of(bool))
-    buffer_output = attr.ib(validator=attr.validators.instance_of(bool))
     required_encryption_context = attr.ib(validator=attr.validators.instance_of(dict))
     required_encryption_context_keys = attr.ib(
         validator=attr.validators.instance_of(list)
     )  # noqa pylint: disable=invalid-name
-    client = attr.ib(validator=attr.validators.instance_of(aws_encryption_sdk.EncryptionSDKClient))
 
-    def __init__(  # noqa pylint: disable=too-many-arguments
+    def __init__(
         self,
         metadata_writer,  # type: MetadataWriter
         interactive,  # type: bool
@@ -180,9 +176,7 @@ class IOHandler(object):
         encode_output,  # type: bool
         required_encryption_context,  # type: Dict[str, str]
         required_encryption_context_keys,  # type: List[str]
-        commitment_policy,  # type: Union[CommitmentPolicy, None]
-        buffer_output,
-        max_encrypted_data_keys,  # type: Union[None, int]
+        commitment_policy,  # type: CommitmentPolicy
     ):
         # type: (...) -> None
         """Workaround pending resolution of attrs/mypy interaction.
@@ -196,20 +190,14 @@ class IOHandler(object):
         self.encode_output = encode_output
         self.required_encryption_context = required_encryption_context
         self.required_encryption_context_keys = required_encryption_context_keys  # pylint: disable=invalid-name
-        if not commitment_policy:
-            commitment_policy = CommitmentPolicy.FORBID_ENCRYPT_ALLOW_DECRYPT
-        self.buffer_output = buffer_output
-        self.client = aws_encryption_sdk.EncryptionSDKClient(
-            commitment_policy=commitment_policy,
-            max_encrypted_data_keys=max_encrypted_data_keys,
-        )
+        self.client = aws_encryption_sdk.EncryptionSDKClient(commitment_policy=commitment_policy)
         attr.validate(self)
 
     def _single_io_write(self, stream_args, source, destination_writer):
         # type: (STREAM_KWARGS, IO, IO) -> OperationResult
         """Performs the actual write operations for a single operation.
 
-        :param dict stream_args: kwargs to pass to `EncryptionSDKClient.stream`
+        :param dict stream_args: kwargs to pass to `aws_encryption_sdk.stream`
         :param source: source to write
         :type source: file-like object
         :param destination_writer: destination object to which to write
@@ -235,7 +223,7 @@ class IOHandler(object):
                 else:
                     metadata_kwargs["header_auth"] = json_ready_header_auth(header_auth)
 
-                if _is_decrypt_mode(str(stream_args["mode"])):
+                if stream_args["mode"] == "decrypt":
                     discovered_ec = handler.header.encryption_context
                     missing_keys = set(self.required_encryption_context_keys).difference(set(discovered_ec.keys()))
                     missing_pairs = set(self.required_encryption_context.items()).difference(set(discovered_ec.items()))
@@ -255,19 +243,16 @@ class IOHandler(object):
                         return OperationResult.FAILED_VALIDATION
 
                 metadata.write_metadata(**metadata_kwargs)
-                if self.buffer_output:
-                    _destination.write(handler.read())
-                else:
-                    for chunk in handler:
-                        _destination.write(chunk)
-                        _destination.flush()
+                for chunk in handler:
+                    _destination.write(chunk)
+                    _destination.flush()
         return OperationResult.SUCCESS
 
     def process_single_operation(self, stream_args, source, destination):
         # type: (STREAM_KWARGS, SOURCE, str) -> OperationResult
         """Processes a single encrypt/decrypt operation given a pre-loaded source.
 
-        :param dict stream_args: kwargs to pass to `EncryptionSDKClient.stream`
+        :param dict stream_args: kwargs to pass to `aws_encryption_sdk.stream`
         :param source: source to write
         :type source: str or file-like object
         :param str destination: destination identifier
@@ -280,7 +265,6 @@ class IOHandler(object):
             if not self._should_write_file(destination):
                 return OperationResult.SKIPPED
             _ensure_dir_exists(destination)
-            # pylint: disable=consider-using-with
             destination_writer = open(os.path.abspath(destination), "wb")
 
         if source == "-":
@@ -332,7 +316,7 @@ class IOHandler(object):
         # type: (STREAM_KWARGS, str, str) -> None
         """Processes a single encrypt/decrypt operation on a source file.
 
-        :param dict stream_args: kwargs to pass to `EncryptionSDKClient.stream`
+        :param dict stream_args: kwargs to pass to `aws_encryption_sdk.stream`
         :param str source: Full file path to source file
         :param str destination: Full file path to destination file
         """
@@ -374,7 +358,7 @@ class IOHandler(object):
         # type: (STREAM_KWARGS, str, str, str) -> None
         """Processes encrypt/decrypt operations on all files in a directory tree.
 
-        :param dict stream_args: kwargs to pass to `EncryptionSDKClient.stream`
+        :param dict stream_args: kwargs to pass to `aws_encryption_sdk.stream`
         :param str source: Full file path to source directory root
         :param str destination: Full file path to destination directory root
         :param str suffix: Suffix to append to output filename
