@@ -733,7 +733,7 @@ def test_cli_unknown_error_capture_stacktrace(patch_process_cli_request, tmpdir,
             + str(tmpdir.join("ciphertext"))
             + " "
             + requested_log_level
-            + " --discovery=true"
+            + " -w discovery=true"
         )
     )
 
